@@ -1,6 +1,7 @@
 import os
 import shutil
 import tempfile
+import time
 from copy import deepcopy
 from pathlib import Path
 from typing import Optional
@@ -23,6 +24,7 @@ from flexlate.path_ops import (
     location_relative_to_new_parent,
 )
 from flexlate.render.multi import MultiRenderer
+from flexlate.styles import SUCCESS_STYLE, INFO_STYLE, console, styled, print_styled
 from flexlate.template.base import Template
 from flexlate.template_data import TemplateData
 from flexlate.transactions.transaction import (
@@ -42,8 +44,11 @@ class Adder:
         target_version: Optional[str] = None,
         out_root: Path = Path("."),
         merged_branch_name: str = DEFAULT_MERGED_BRANCH_NAME,
+        base_merged_branch_name: str = DEFAULT_MERGED_BRANCH_NAME,
         template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
+        base_template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
         add_mode: AddMode = AddMode.LOCAL,
+        remote: str = "origin",
         config_manager: ConfigManager = ConfigManager(),
     ):
         assert_repo_is_in_clean_state(repo)
@@ -69,39 +74,52 @@ class Adder:
                 os.path.relpath(template.path.resolve(), config_path.parent.resolve())
             )
 
-        if add_mode == AddMode.USER:
-            # No need to use git if adding for user
-            config_manager.add_template_source(
-                template,
-                config_path,
-                target_version=target_version,
-                project_root=Path(repo.working_dir),  # type: ignore
+        with console.status(print_styled("Adding template source...", INFO_STYLE)):
+            print_styled(
+                f"Adding template source {template.name} from {template.git_url or template.path}",
+                INFO_STYLE,
             )
-            return
 
-        commit_message = create_transaction_commit_message(
-            _add_template_source_commit_message(
-                template, out_root, Path(repo.working_dir)
-            ),
-            transaction,
-        )
+            if add_mode == AddMode.USER:
+                # No need to use git if adding for user
+                config_manager.add_template_source(
+                    template,
+                    config_path,
+                    target_version=target_version,
+                    project_root=Path(repo.working_dir),  # type: ignore
+                )
+                return
 
-        # Local or project config, add in git
-        modify_files_via_branches_and_temp_repo(
-            lambda temp_path: config_manager.add_template_source(
-                template,
-                location_relative_to_new_parent(
-                    config_path, project_root, temp_path, Path(os.getcwd())
+            commit_message = create_transaction_commit_message(
+                _add_template_source_commit_message(
+                    template, out_root, Path(repo.working_dir)
                 ),
-                target_version=target_version,
-                project_root=temp_path,
-            ),
-            repo,
-            commit_message,
-            out_root,
-            merged_branch_name=merged_branch_name,
-            template_branch_name=template_branch_name,
-        )
+                transaction,
+            )
+
+            # Local or project config, add in git
+            modify_files_via_branches_and_temp_repo(
+                lambda temp_path: config_manager.add_template_source(
+                    template,
+                    location_relative_to_new_parent(
+                        config_path, project_root, temp_path, Path(os.getcwd())
+                    ),
+                    target_version=target_version,
+                    project_root=temp_path,
+                ),
+                repo,
+                commit_message,
+                out_root,
+                merged_branch_name=merged_branch_name,
+                base_merged_branch_name=base_merged_branch_name,
+                template_branch_name=template_branch_name,
+                base_template_branch_name=base_template_branch_name,
+                remote=remote,
+            )
+
+            print_styled(
+                f"Sucessfully added template source {template.name}", INFO_STYLE
+            )
 
     def apply_template_and_add(
         self,
@@ -112,8 +130,11 @@ class Adder:
         out_root: Path = Path("."),
         add_mode: AddMode = AddMode.LOCAL,
         merged_branch_name: str = DEFAULT_MERGED_BRANCH_NAME,
+        base_merged_branch_name: str = DEFAULT_MERGED_BRANCH_NAME,
         template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
+        base_template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
         no_input: bool = False,
+        remote: str = "origin",
         config_manager: ConfigManager = ConfigManager(),
         updater: Updater = Updater(),
         renderer: MultiRenderer = MultiRenderer(),
@@ -131,6 +152,11 @@ class Adder:
             out_root, project_root, template.render_relative_root_in_output, add_mode
         )
 
+        print_styled(
+            f"Applying template {template.name} to {expanded_out_root.resolve()}",
+            INFO_STYLE,
+        )
+
         if add_mode == AddMode.USER:
             # No need to commit config changes for user
             config_manager.add_applied_template(
@@ -165,7 +191,10 @@ class Adder:
                 commit_message,
                 out_root,
                 merged_branch_name=merged_branch_name,
+                base_merged_branch_name=base_merged_branch_name,
                 template_branch_name=template_branch_name,
+                base_template_branch_name=base_template_branch_name,
+                remote=remote,
             )
         template_update = TemplateUpdate(
             template=template,
@@ -181,12 +210,18 @@ class Adder:
             [template_update],
             transaction,
             merged_branch_name=merged_branch_name,
+            base_merged_branch_name=base_merged_branch_name,
             template_branch_name=template_branch_name,
+            base_template_branch_name=base_template_branch_name,
             no_input=no_input,
             full_rerender=False,
             renderer=renderer,
             config_manager=config_manager,
         )
+        print_styled(
+            f"Successfully applied template {template.name} to {expanded_out_root.resolve()}",
+            SUCCESS_STYLE,
+        )
 
     def init_project_and_add_to_branches(
         self,
@@ -195,6 +230,7 @@ class Adder:
         merged_branch_name: str = DEFAULT_MERGED_BRANCH_NAME,
         template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
         user: bool = False,
+        remote: str = "origin",
         config_manager: ConfigManager = ConfigManager(),
     ):
         assert_repo_is_in_clean_state(repo)
@@ -203,31 +239,49 @@ class Adder:
 
         path = Path(repo.working_dir)
 
-        if user:
-            # Simply init the project for the user
-            config_manager.add_project(
-                path=path,
-                default_add_mode=default_add_mode,
-                user=user,
-                merged_branch_name=merged_branch_name,
-                template_branch_name=template_branch_name,
+        with console.status(styled("Initializing...", INFO_STYLE)):
+            print_styled(
+                f"Initializing flexlate project with default add mode "
+                f"{default_add_mode.value} and user={user} in "
+                f"{Path(repo.working_dir).resolve()}",
+                INFO_STYLE,
             )
-            return
 
-        # Config resides in project, so add it via branches
-        modify_files_via_branches_and_temp_repo(
-            lambda temp_path: config_manager.add_project(
-                path=temp_path,
-                default_add_mode=default_add_mode,
-                user=user,
+            if user:
+                # Simply init the project for the user
+                config_manager.add_project(
+                    path=path,
+                    default_add_mode=default_add_mode,
+                    user=user,
+                    merged_branch_name=merged_branch_name,
+                    template_branch_name=template_branch_name,
+                    remote=remote,
+                )
+                return
+
+            # Config resides in project, so add it via branches
+            modify_files_via_branches_and_temp_repo(
+                lambda temp_path: config_manager.add_project(
+                    path=temp_path,
+                    default_add_mode=default_add_mode,
+                    user=user,
+                    merged_branch_name=merged_branch_name,
+                    template_branch_name=template_branch_name,
+                    remote=remote,
+                ),
+                repo,
+                "Initialized flexlate project",
+                path,
                 merged_branch_name=merged_branch_name,
+                base_merged_branch_name=merged_branch_name,
                 template_branch_name=template_branch_name,
-            ),
-            repo,
-            "Initialized flexlate project",
-            path,
-            merged_branch_name=merged_branch_name,
-            template_branch_name=template_branch_name,
+                base_template_branch_name=template_branch_name,
+                remote=remote,
+            )
+
+        print_styled(
+            f"Finished initializing flexlate project",
+            SUCCESS_STYLE,
         )
 
     def init_project_from_template_source_path(
@@ -241,6 +295,7 @@ class Adder:
         default_add_mode: AddMode = AddMode.LOCAL,
         merged_branch_name: str = DEFAULT_MERGED_BRANCH_NAME,
         template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
+        remote: str = "origin",
         no_input: bool = False,
         config_manager: ConfigManager = ConfigManager(),
         updater: Updater = Updater(),
@@ -258,6 +313,7 @@ class Adder:
                 merged_branch_name=merged_branch_name,
                 template_branch_name=template_branch_name,
                 user=False,
+                remote=remote,
                 config_manager=config_manager,
             )
             self.add_template_source(
@@ -269,6 +325,7 @@ class Adder:
                 merged_branch_name=merged_branch_name,
                 template_branch_name=template_branch_name,
                 add_mode=AddMode.LOCAL,
+                remote=remote,
                 config_manager=config_manager,
             )
             self.apply_template_and_add(
@@ -281,6 +338,7 @@ class Adder:
                 merged_branch_name=merged_branch_name,
                 template_branch_name=template_branch_name,
                 no_input=no_input,
+                remote=remote,
                 config_manager=config_manager,
                 updater=updater,
                 renderer=renderer,
