@@ -5,11 +5,15 @@ from git import Repo
 
 from flexlate.adder import Adder
 from flexlate.add_mode import AddMode
+from flexlate.branch_update import get_flexlate_branch_name
 from flexlate.config_manager import ConfigManager
 from flexlate.constants import DEFAULT_MERGED_BRANCH_NAME, DEFAULT_TEMPLATE_BRANCH_NAME
 from flexlate.finder.multi import MultiFinder
+from flexlate.merger import Merger
+from flexlate.pusher import Pusher
 from flexlate.remover import Remover
 from flexlate.render.multi import MultiRenderer
+from flexlate.styles import console
 from flexlate.syncer import Syncer
 from flexlate.template.base import Template
 from flexlate.template_data import TemplateData
@@ -21,19 +25,28 @@ from flexlate.update.main import Updater
 class Flexlate:
     def __init__(
         self,
+        quiet: bool = False,
         adder: Adder = Adder(),
         remover: Remover = Remover(),
         config_manager: ConfigManager = ConfigManager(),
+        merger: Merger = Merger(),
         finder: MultiFinder = MultiFinder(),
+        pusher: Pusher = Pusher(),
         renderer: MultiRenderer = MultiRenderer(),
         syncer: Syncer = Syncer(),
         updater: Updater = Updater(),
         undoer: Undoer = Undoer(),
     ):
+        self.quiet = quiet
+        # Let rich handle suppressing output
+        console.quiet = quiet
+
         self.adder = adder
         self.remover = remover
         self.config_manager = config_manager
+        self.merger = merger
         self.finder = finder
+        self.pusher = pusher
         self.renderer = renderer
         self.syncer = syncer
         self.updater = updater
@@ -47,6 +60,7 @@ class Flexlate:
         template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
         template_version: Optional[str] = None,
         user: bool = False,
+        remote: str = "origin",
     ):
         repo = Repo(path)
         self.adder.init_project_and_add_to_branches(
@@ -55,6 +69,7 @@ class Flexlate:
             user=user,
             merged_branch_name=merged_branch_name,
             template_branch_name=template_branch_name,
+            remote=remote,
         )
 
     def init_project_from(
@@ -66,6 +81,7 @@ class Flexlate:
         default_folder_name: str = "project",
         no_input: bool = False,
         default_add_mode: AddMode = AddMode.LOCAL,
+        remote: str = "origin",
         merged_branch_name: str = DEFAULT_MERGED_BRANCH_NAME,
         template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
     ):
@@ -83,6 +99,7 @@ class Flexlate:
             data=data,
             no_input=no_input,
             default_add_mode=default_add_mode,
+            remote=remote,
             merged_branch_name=merged_branch_name,
             template_branch_name=template_branch_name,
         )
@@ -110,9 +127,16 @@ class Flexlate:
             transaction,
             target_version=target_version,
             out_root=template_root,
-            merged_branch_name=project_config.merged_branch_name,
-            template_branch_name=project_config.template_branch_name,
+            merged_branch_name=get_flexlate_branch_name(
+                repo, project_config.merged_branch_name
+            ),
+            base_merged_branch_name=project_config.merged_branch_name,
+            template_branch_name=get_flexlate_branch_name(
+                repo, project_config.template_branch_name
+            ),
+            base_template_branch_name=project_config.template_branch_name,
             add_mode=add_mode,
+            remote=project_config.remote,
             config_manager=self.config_manager,
         )
 
@@ -133,9 +157,16 @@ class Flexlate:
             template_name,
             transaction,
             out_root=template_root,
-            merged_branch_name=project_config.merged_branch_name,
-            template_branch_name=project_config.template_branch_name,
+            merged_branch_name=get_flexlate_branch_name(
+                repo, project_config.merged_branch_name
+            ),
+            base_merged_branch_name=project_config.merged_branch_name,
+            template_branch_name=get_flexlate_branch_name(
+                repo, project_config.template_branch_name
+            ),
+            base_template_branch_name=project_config.template_branch_name,
             add_mode=project_config.default_add_mode,
+            remote=project_config.remote,
             config_manager=self.config_manager,
         )
 
@@ -164,8 +195,15 @@ class Flexlate:
             out_root=out_root,
             add_mode=add_mode,
             no_input=no_input,
-            merged_branch_name=project_config.merged_branch_name,
-            template_branch_name=project_config.template_branch_name,
+            merged_branch_name=get_flexlate_branch_name(
+                repo, project_config.merged_branch_name
+            ),
+            base_merged_branch_name=project_config.merged_branch_name,
+            template_branch_name=get_flexlate_branch_name(
+                repo, project_config.template_branch_name
+            ),
+            base_template_branch_name=project_config.template_branch_name,
+            remote=project_config.remote,
             config_manager=self.config_manager,
             renderer=self.renderer,
             updater=self.updater,
@@ -188,8 +226,15 @@ class Flexlate:
             transaction,
             out_root=out_root,
             add_mode=project_config.default_add_mode,
-            merged_branch_name=project_config.merged_branch_name,
-            template_branch_name=project_config.template_branch_name,
+            merged_branch_name=get_flexlate_branch_name(
+                repo, project_config.merged_branch_name
+            ),
+            base_merged_branch_name=project_config.merged_branch_name,
+            template_branch_name=get_flexlate_branch_name(
+                repo, project_config.template_branch_name
+            ),
+            base_template_branch_name=project_config.template_branch_name,
+            remote=project_config.remote,
             config_manager=self.config_manager,
             updater=self.updater,
             renderer=self.renderer,
@@ -200,6 +245,7 @@ class Flexlate:
         names: Optional[List[str]] = None,
         data: Optional[Sequence[TemplateData]] = None,
         no_input: bool = False,
+        abort_on_conflict: bool = False,
         project_path: Path = Path("."),
     ):
         transaction = FlexlateTransaction(
@@ -242,8 +288,15 @@ class Flexlate:
             updates,
             transaction,
             no_input=no_input,
-            merged_branch_name=project_config.merged_branch_name,
-            template_branch_name=project_config.template_branch_name,
+            abort_on_conflict=abort_on_conflict,
+            merged_branch_name=get_flexlate_branch_name(
+                repo, project_config.merged_branch_name
+            ),
+            base_merged_branch_name=project_config.merged_branch_name,
+            template_branch_name=get_flexlate_branch_name(
+                repo, project_config.template_branch_name
+            ),
+            base_template_branch_name=project_config.template_branch_name,
             renderer=self.renderer,
             config_manager=self.config_manager,
         )
@@ -254,11 +307,21 @@ class Flexlate:
         self.undoer.undo_transactions(
             repo,
             num_transactions=num_operations,
-            merged_branch_name=project_config.merged_branch_name,
-            template_branch_name=project_config.template_branch_name,
+            merged_branch_name=get_flexlate_branch_name(
+                repo, project_config.merged_branch_name
+            ),
+            base_merged_branch_name=project_config.merged_branch_name,
+            template_branch_name=get_flexlate_branch_name(
+                repo, project_config.template_branch_name
+            ),
+            base_template_branch_name=project_config.template_branch_name,
         )
 
-    def sync(self, no_input: bool = False, project_path: Path = Path(".")):
+    def sync(
+        self,
+        prompt: bool = False,
+        project_path: Path = Path("."),
+    ):
         project_config = self.config_manager.load_project_config(project_path)
         repo = Repo(project_config.path)
         transaction = FlexlateTransaction(
@@ -267,13 +330,66 @@ class Flexlate:
         self.syncer.sync_local_changes_to_flexlate_branches(
             repo,
             transaction,
-            merged_branch_name=project_config.merged_branch_name,
-            template_branch_name=project_config.template_branch_name,
-            no_input=no_input,
+            merged_branch_name=get_flexlate_branch_name(
+                repo, project_config.merged_branch_name
+            ),
+            base_merged_branch_name=project_config.merged_branch_name,
+            template_branch_name=get_flexlate_branch_name(
+                repo, project_config.template_branch_name
+            ),
+            base_template_branch_name=project_config.template_branch_name,
+            no_input=not prompt,
             updater=self.updater,
             renderer=self.renderer,
             config_manager=self.config_manager,
         )
 
+    def merge_flexlate_branches(
+        self,
+        branch_name: Optional[str] = None,
+        delete: bool = True,
+        project_path: Path = Path("."),
+    ):
+        project_config = self.config_manager.load_project_config(project_path)
+        repo = Repo(project_config.path)
+        self.merger.merge_flexlate_branches(
+            repo,
+            branch_name,
+            delete=delete,
+            merged_branch_name=project_config.merged_branch_name,
+            template_branch_name=project_config.template_branch_name,
+        )
+
+    def push_main_flexlate_branches(
+        self,
+        remote: Optional[str] = None,
+        project_path: Path = Path("."),
+    ):
+        project_config = self.config_manager.load_project_config(project_path)
+        use_remote = remote or project_config.remote
+        repo = Repo(project_config.path)
+        self.pusher.push_main_flexlate_branches(
+            repo,
+            use_remote,
+            merged_branch_name=project_config.merged_branch_name,
+            template_branch_name=project_config.template_branch_name,
+        )
+
+    def push_feature_flexlate_branches(
+        self,
+        feature_branch: Optional[str] = None,
+        remote: str = "origin",
+        project_path: Path = Path("."),
+    ):
+        project_config = self.config_manager.load_project_config(project_path)
+        repo = Repo(project_config.path)
+        self.pusher.push_feature_flexlate_branches(
+            repo,
+            feature_branch,
+            remote,
+            merged_branch_name=project_config.merged_branch_name,
+            template_branch_name=project_config.template_branch_name,
+        )
+
     # TODO: list template sources, list applied templates
     # TODO: Update target versions in template sources
