@@ -3,6 +3,7 @@ from git import Repo
 from flexlate.config_manager import ConfigManager
 from flexlate.constants import DEFAULT_TEMPLATE_BRANCH_NAME, DEFAULT_MERGED_BRANCH_NAME
 from flexlate.render.multi import MultiRenderer
+from flexlate.styles import print_styled, INFO_STYLE, SUCCESS_STYLE
 from flexlate.transactions.transaction import FlexlateTransaction
 from flexlate.update.main import Updater
 
@@ -13,20 +14,28 @@ class Syncer:
         repo: Repo,
         transaction: FlexlateTransaction,
         merged_branch_name: str = DEFAULT_MERGED_BRANCH_NAME,
+        base_merged_branch_name: str = DEFAULT_MERGED_BRANCH_NAME,
         template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
+        base_template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
         no_input: bool = False,
         updater: Updater = Updater(),
         renderer: MultiRenderer = MultiRenderer(),
         config_manager: ConfigManager = ConfigManager(),
     ):
+        print_styled("Syncing local changes to flexlate branches", INFO_STYLE)
         updater.update(
             repo,
             [],
             transaction,
             merged_branch_name=merged_branch_name,
+            base_merged_branch_name=base_merged_branch_name,
             template_branch_name=template_branch_name,
+            base_template_branch_name=base_template_branch_name,
             no_input=no_input,
             full_rerender=True,
             renderer=renderer,
             config_manager=config_manager,
         )
+        print_styled(
+            "Successfully synced local changes to flexlate branches", SUCCESS_STYLE
+        )
