@@ -1,10 +1,12 @@
 import os
+from copy import deepcopy
 from pathlib import Path
 from typing import Sequence, Optional, List, Dict, Any
 
 from git import Repo
 from rich.prompt import Confirm
 
+from flexlate.branch_update import abort_merge_and_reset_flexlate_branches
 from flexlate.cli_utils import confirm_user
 from flexlate.config_manager import ConfigManager
 from flexlate.constants import DEFAULT_MERGED_BRANCH_NAME, DEFAULT_TEMPLATE_BRANCH_NAME
@@ -22,6 +24,8 @@ from flexlate.styles import (
     styled,
     QUESTION_STYLE,
     ALERT_STYLE,
+    INFO_STYLE,
+    SUCCESS_STYLE,
 )
 from flexlate.template.base import Template
 from flexlate.ext_git import (
@@ -54,8 +58,11 @@ class Updater:
         updates: Sequence[TemplateUpdate],
         transaction: FlexlateTransaction,
         merged_branch_name: str = DEFAULT_MERGED_BRANCH_NAME,
+        base_merged_branch_name: str = DEFAULT_MERGED_BRANCH_NAME,
         template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
+        base_template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
         no_input: bool = False,
+        abort_on_conflict: bool = False,
         full_rerender: bool = True,
         renderer: MultiRenderer = MultiRenderer(),
         config_manager: ConfigManager = ConfigManager(),
@@ -76,7 +83,10 @@ class Updater:
         # Create it from the initial commit if it does not exist
         cwd = Path(os.getcwd())
         with temp_repo_that_pushes_to_branch(  # type: ignore
-            repo, branch_name=template_branch_name, delete_tracked_files=full_rerender
+            repo,
+            branch_name=template_branch_name,
+            base_branch_name=base_template_branch_name,
+            delete_tracked_files=full_rerender,
         ) as temp_repo:
             temp_project_root = Path(temp_repo.working_dir)  # type: ignore
             temp_updates = _move_update_config_locations_to_new_parent(
@@ -91,12 +101,23 @@ class Updater:
                 use_template_source_path=False,
             )
             orig_renderables = (
-                config_manager.get_all_renderables(project_root=temp_project_root)
+                config_manager.get_all_renderables(
+                    relative_to=project_root, project_root=temp_project_root
+                )
                 if full_rerender
                 else config_manager.get_renderables_for_updates(
                     updates, project_root=project_root
                 )
             )
+            if full_rerender:
+                print_styled(
+                    f"Syncing changes in flexlate configs to output", INFO_STYLE
+                )
+            else:
+                print_styled(
+                    f"Updating {len(orig_renderables)} applied templates", INFO_STYLE
+                )
+
             renderables = _move_renderable_out_roots_to_new_parent(
                 orig_renderables,
                 project_root,
@@ -141,14 +162,23 @@ class Updater:
             repo, merged_branch_name, current_branch.name
         )
         # Update with template changes
-        checkout_template_branch(repo, merged_branch_name)
+        checkout_template_branch(repo, merged_branch_name, base_merged_branch_name)
         merge_branch_into_current(repo, template_branch_name)
 
         if repo_has_merge_conflicts(repo):
-            if no_input:
-                # Not receiving input, so there is no way the user could resolve conflicts
-                # during the process
-                print_styled("Repo has merge conflicts after update", ALERT_STYLE)
+            if abort_on_conflict:
+                print_styled(
+                    "Repo has merge conflicts after update, aborting due to abort_on_conflict=True",
+                    ALERT_STYLE,
+                )
+                abort_merge_and_reset_flexlate_branches(
+                    repo,
+                    current_branch,
+                    merged_branch_sha=merged_branch_sha,
+                    template_branch_sha=template_branch_sha,
+                    merged_branch_name=merged_branch_name,
+                    template_branch_name=template_branch_name,
+                )
                 return
 
             # Need to wait for user to resolve merge conflicts
@@ -164,14 +194,13 @@ class Updater:
                     "Aborting update.",
                     ALERT_STYLE,
                 )
-                # Abort merge and reset flexlate branches to original state
-                abort_merge(repo)
-                current_branch.checkout()
-                reset_branch_to_commit_without_checkout(
-                    repo, merged_branch_name, merged_branch_sha
-                )
-                reset_branch_to_commit_without_checkout(
-                    repo, template_branch_name, template_branch_sha
+                abort_merge_and_reset_flexlate_branches(
+                    repo,
+                    current_branch,
+                    merged_branch_sha=merged_branch_sha,
+                    template_branch_sha=template_branch_sha,
+                    merged_branch_name=merged_branch_name,
+                    template_branch_name=template_branch_name,
                 )
                 return
 
@@ -196,6 +225,8 @@ class Updater:
         # need to set cwd again
         os.chdir(cwd)
 
+        print_styled("Successfully updated template output", SUCCESS_STYLE)
+
     def get_updates_for_templates(
         self,
         templates: Sequence[Template],
@@ -283,7 +314,7 @@ def _move_update_config_locations_to_new_parent(
 
 
 def _move_renderable_out_roots_to_new_parent(
-    renderbales: Sequence[Renderable], orig_parent: Path, new_parent: Path
+    renderables: Sequence[Renderable], orig_parent: Path, new_parent: Path
 ) -> List[Renderable]:
     return [
         renderable.copy(
@@ -293,5 +324,5 @@ def _move_renderable_out_roots_to_new_parent(
                 )
             )
         )
-        for renderable in renderbales
+        for renderable in renderables
     ]
