@@ -10,6 +10,7 @@ from typing import List, Final, Callable, Optional, ContextManager
 
 import pytest
 
+from flexlate.add_mode import AddMode
 from flexlate.template.types import TemplateType
 from flexlate.template_data import TemplateData
 from tests.config import (
@@ -47,12 +48,19 @@ class TemplateSourceFixture:
     name: str
     path: str
     type: TemplateSourceType
+    template_type: TemplateType
     input_data: TemplateData
     update_input_data: TemplateData
     version_1: str
     version_2: str
     is_local_template: bool = False
     version_migrate_func: Callable[[str], None] = lambda path: None
+    render_relative_root_in_output: Path = Path(".")
+    render_relative_root_in_template: Path = Path(".")
+    expect_local_applied_template_path: Path = Path(".")
+    evaluated_render_relative_root_in_output_creator: Callable[
+        [TemplateData], Path
+    ] = lambda data: Path(".")
 
     @property
     def default_version(self) -> str:
@@ -68,13 +76,6 @@ class TemplateSourceFixture:
             return None
         return self.path
 
-    def expect_path(self, version: Optional[str] = None) -> str:
-        version = version or self.default_version
-        if self.is_local_template:
-            return self.path
-        # appdirs user data directory is mocked to GENERATED_FILES_DIR in tests
-        return str(GENERATED_FILES_DIR / self.name / version)
-
     def relative(self, to: Path) -> "TemplateSourceFixture":
         new_fixture = deepcopy(self)
         if not self.is_local_template:
@@ -97,37 +98,51 @@ COOKIECUTTER_REMOTE_FIXTURE: Final[TemplateSourceFixture] = TemplateSourceFixtur
     name=COOKIECUTTER_REMOTE_NAME,
     path=COOKIECUTTER_REMOTE_URL,
     type=TemplateSourceType.COOKIECUTTER_REMOTE,
+    template_type=TemplateType.COOKIECUTTER,
     input_data=dict(name="woo", key="it works"),
     update_input_data=dict(name="updated", key="now"),
     version_1=COOKIECUTTER_REMOTE_VERSION_1,
     version_2=COOKIECUTTER_REMOTE_VERSION_2,
+    render_relative_root_in_output=Path("{{ cookiecutter.name }}"),
+    render_relative_root_in_template=Path("{{ cookiecutter.name }}"),
+    evaluated_render_relative_root_in_output_creator=lambda data: Path(data["name"]),
+    expect_local_applied_template_path=Path(".."),
 )
 copier_remote_fixture: Final[TemplateSourceFixture] = TemplateSourceFixture(
     name=COPIER_REMOTE_NAME,
     path=COPIER_REMOTE_URL,
     type=TemplateSourceType.COPIER_REMOTE,
+    template_type=TemplateType.COPIER,
     input_data=dict(question1="oh yeah", question2=10.5),
     update_input_data=dict(question1="please work", question2=2.8),
     version_1=COPIER_REMOTE_VERSION_1,
     version_2=COPIER_REMOTE_VERSION_2,
+    render_relative_root_in_output=Path("."),
+    render_relative_root_in_template=Path("output"),
 )
 
 cookiecutter_local_fixture: Final[TemplateSourceFixture] = TemplateSourceFixture(
     name=COOKIECUTTER_ONE_NAME,
     path=COOKIECUTTER_ONE_DIR,
     type=TemplateSourceType.COOKIECUTTER_LOCAL,
+    template_type=TemplateType.COOKIECUTTER,
     input_data=dict(a="z", c="f"),
     update_input_data=dict(a="n", c="q"),
     version_1=COOKIECUTTER_ONE_VERSION,
     version_2=COOKIECUTTER_ONE_MODIFIED_VERSION,
     is_local_template=True,
     version_migrate_func=modify_cookiecutter_one,
+    render_relative_root_in_output=Path("{{ cookiecutter.a }}"),
+    render_relative_root_in_template=Path("{{ cookiecutter.a }}"),
+    evaluated_render_relative_root_in_output_creator=lambda data: Path(data["a"]),
+    expect_local_applied_template_path=Path(".."),
 )
 
 copier_local_fixture: Final[TemplateSourceFixture] = TemplateSourceFixture(
     name=COPIER_ONE_NAME,
     path=COPIER_ONE_DIR,
     type=TemplateSourceType.COPIER_LOCAL,
+    template_type=TemplateType.COPIER,
     input_data=dict(q1="abc", q2=2, q3="def"),
     update_input_data=dict(q1="qwe", q2=2, q3="rty"),
     version_1=COPIER_ONE_VERSION,
@@ -217,4 +232,4 @@ def template_source_one_remote_and_all_local_relative(request) -> TemplateSource
     yield request.param
 
 
-COOKIECUTTER_REMOTE_DEFAULT_EXPECT_PATH = COOKIECUTTER_REMOTE_FIXTURE.expect_path()
+COOKIECUTTER_REMOTE_DEFAULT_EXPECT_PATH = COOKIECUTTER_REMOTE_FIXTURE.path
