@@ -2,6 +2,8 @@ import pytest
 
 from flexlate.add_mode import AddMode
 from flexlate.adder import Adder
+from flexlate.branch_update import get_flexlate_branch_name
+from flexlate.constants import DEFAULT_MERGED_BRANCH_NAME, DEFAULT_TEMPLATE_BRANCH_NAME
 from flexlate.remover import Remover
 from flexlate.transactions.transaction import FlexlateTransaction
 from flexlate.update.main import Updater
@@ -17,6 +19,7 @@ from tests.fixtures.transaction import (
     remove_output_transaction,
     update_transaction,
 )
+from tests.gitutils import rename_branch
 
 
 @pytest.fixture
@@ -97,6 +100,31 @@ def repo_with_gitignore_and_cookiecutter_one_template_source(
     yield repo
 
 
+@pytest.fixture
+def repo_with_gitignore_and_cookiecutter_one_template_source_in_repo(
+    repo_with_gitignore: Repo,
+    cookiecutter_one_template_in_repo: CookiecutterTemplate,
+    add_source_transaction: FlexlateTransaction,
+) -> Repo:
+    repo = repo_with_gitignore
+    # While the same changes are in cookiecutter_one_template_in_repo, they do not persist,
+    # so make them again
+    new_folder = GENERATED_REPO_DIR / "templates"
+    new_folder.mkdir(parents=True)
+    new_path = new_folder / COOKIECUTTER_ONE_DIR.name
+    shutil.copytree(COOKIECUTTER_ONE_DIR, new_path)
+    stage_and_commit_all(repo, "Add cookiecutter one template in repo")
+    with change_directory_to(GENERATED_REPO_DIR):
+        adder = Adder()
+        adder.add_template_source(
+            repo,
+            cookiecutter_one_template_in_repo,
+            add_source_transaction,
+            out_root=GENERATED_REPO_DIR,
+        )
+    yield repo
+
+
 @pytest.fixture
 def repo_with_remote_cookiecutter_template_source(
     repo_with_placeholder_committed: Repo,
@@ -134,6 +162,23 @@ def repo_with_template_branch_from_cookiecutter_one(
     yield repo
 
 
+@pytest.fixture
+def repo_with_template_branch_from_cookiecutter_one_and_feature_flexlate_branches(
+    repo_with_template_branch_from_cookiecutter_one: Repo,
+) -> Repo:
+    repo = repo_with_template_branch_from_cookiecutter_one
+    # Simulate conditions of developing on feature branch
+    feature_merged_branch_name = get_flexlate_branch_name(
+        repo, DEFAULT_MERGED_BRANCH_NAME
+    )
+    feature_template_branch_name = get_flexlate_branch_name(
+        repo, DEFAULT_TEMPLATE_BRANCH_NAME
+    )
+    rename_branch(repo, DEFAULT_MERGED_BRANCH_NAME, feature_merged_branch_name)
+    rename_branch(repo, DEFAULT_TEMPLATE_BRANCH_NAME, feature_template_branch_name)
+    yield repo
+
+
 @pytest.fixture
 def repo_with_template_branch_from_cookiecutter_one_project_add_mode(
     repo_with_cookiecutter_one_template_source: Repo,
@@ -193,6 +238,25 @@ def repo_with_gitignore_and_template_branch_from_cookiecutter_one(
     yield repo
 
 
+@pytest.fixture
+def repo_with_gitignore_and_template_branch_from_cookiecutter_one_in_repo(
+    repo_with_gitignore_and_cookiecutter_one_template_source_in_repo: Repo,
+    cookiecutter_one_template_in_repo: CookiecutterTemplate,
+    add_output_transaction: FlexlateTransaction,
+) -> Repo:
+    repo = repo_with_gitignore_and_cookiecutter_one_template_source_in_repo
+    with change_directory_to(GENERATED_REPO_DIR):
+        adder = Adder()
+        adder.apply_template_and_add(
+            repo,
+            cookiecutter_one_template_in_repo,
+            add_output_transaction,
+            out_root=GENERATED_REPO_DIR,
+            no_input=True,
+        )
+    yield repo
+
+
 @pytest.fixture
 def repo_from_cookiecutter_one_with_modifications(
     repo_with_template_branch_from_cookiecutter_one: Repo,
