@@ -1,8 +1,9 @@
 from pathlib import Path
+from typing import Union
 
 from git import Repo
 
-from tests.config import GENERATED_REPO_DIR
+from tests.config import GENERATED_REPO_DIR, GENERATED_FILES_DIR
 
 
 def create_empty_repo(out_dir: Path = GENERATED_REPO_DIR) -> Repo:
@@ -42,5 +43,35 @@ def assert_main_commit_message_matches(message1: str, message2: str):
     assert main_1 == main_2
 
 
+def rename_branch(repo: Repo, branch_name: str, new_branch_name: str):
+    repo.git.branch("-m", branch_name, new_branch_name)
+
+
+def checkout_new_branch(repo: Repo, branch_name: str):
+    repo.git.checkout("-b", branch_name)
+
+
+def checkout_existing_branch(repo: Repo, branch_name: str):
+    repo.git.checkout(branch_name)
+
+
+def add_remote(repo: Repo, path: Union[str, Path], remote_name: str = "origin"):
+    if isinstance(path, Path):
+        remote_path = str(path.resolve())
+    else:
+        remote_path = path
+    repo.git.remote("add", remote_name, remote_path)
+
+
+def add_local_remote(
+    repo: Repo, remote_path: Path = GENERATED_FILES_DIR / "remote"
+) -> Repo:
+    if not remote_path.exists():
+        remote_path.mkdir()
+    remote_repo = Repo.init(remote_path)
+    add_remote(repo, remote_path)
+    return remote_repo
+
+
 def _get_main_message_from_commit_message(message: str) -> str:
     return message.split("\n")[0]
