@@ -58,6 +58,7 @@ class CLIStubFlexlate(Flexlate):
         template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
         template_path_from: Optional[str] = None,
         user: bool = False,
+        remote: str = "origin",
     ):
         fxt(
             [
@@ -71,6 +72,8 @@ class CLIStubFlexlate(Flexlate):
                 template_branch_name,
                 *_bool_flag(user, "user"),
                 *_flag_if_not_none(template_path_from, "from"),
+                "--remote",
+                remote,
             ]
         )
 
@@ -82,7 +85,9 @@ class CLIStubFlexlate(Flexlate):
         data: Optional[TemplateData] = None,
         default_folder_name: str = "project",
         no_input: bool = False,
+        quiet: bool = False,
         default_add_mode: AddMode = AddMode.LOCAL,
+        remote: str = "origin",
         merged_branch_name: str = DEFAULT_MERGED_BRANCH_NAME,
         template_branch_name: str = DEFAULT_TEMPLATE_BRANCH_NAME,
     ):
@@ -99,12 +104,15 @@ class CLIStubFlexlate(Flexlate):
                 "--folder-name",
                 default_folder_name,
                 *_bool_flag(no_input, "no-input"),
+                *_bool_flag(quiet, "quiet"),
                 "--add-mode",
                 default_add_mode.value,
                 "--merged-branch-name",
                 merged_branch_name,
                 "--template-branch-name",
                 template_branch_name,
+                "--remote",
+                remote,
             ],
             input_data=all_data,
         )
@@ -136,8 +144,18 @@ class CLIStubFlexlate(Flexlate):
         self,
         template_name: str,
         template_root: Path = Path("."),
+        quiet: bool = False,
     ):
-        fxt(["remove", "source", template_name, "--root", str(template_root)])
+        fxt(
+            [
+                "remove",
+                "source",
+                template_name,
+                "--root",
+                str(template_root),
+                *_bool_flag(quiet, "quiet"),
+            ]
+        )
 
     def apply_template_and_add(
         self,
@@ -146,15 +164,16 @@ class CLIStubFlexlate(Flexlate):
         out_root: Path = Path("."),
         add_mode: Optional[AddMode] = None,
         no_input: bool = False,
+        quiet: bool = False,
     ):
         fxt(
             [
                 "add",
                 "output",
                 name,
-                "--root",
                 str(out_root),
                 *_bool_flag(no_input, "no-input"),
+                *_bool_flag(quiet, "quiet"),
                 *_flag_if_not_none(
                     add_mode.value if add_mode is not None else None, "add-mode"
                 ),
@@ -163,17 +182,25 @@ class CLIStubFlexlate(Flexlate):
         )
 
     def remove_applied_template_and_output(
-        self,
-        template_name: str,
-        out_root: Path = Path("."),
+        self, template_name: str, out_root: Path = Path("."), quiet: bool = False
     ):
-        fxt(["remove", "output", template_name, "--root", str(out_root)])
+        fxt(
+            [
+                "remove",
+                "output",
+                template_name,
+                str(out_root),
+                *_bool_flag(quiet, "quiet"),
+            ]
+        )
 
     def update(
         self,
         names: Optional[List[str]] = None,
         data: Optional[Sequence[TemplateData]] = None,
         no_input: bool = False,
+        abort_on_conflict: bool = False,
+        quiet: bool = False,
         project_path: Path = Path("."),
     ):
         fxt(
@@ -183,6 +210,8 @@ class CLIStubFlexlate(Flexlate):
                 "--path",
                 str(project_path),
                 *_bool_flag(no_input, "no-input"),
+                *_bool_flag(quiet, "quiet"),
+                *_bool_flag(abort_on_conflict, "abort"),
             ],
             input_data=data,
         )
@@ -190,8 +219,61 @@ class CLIStubFlexlate(Flexlate):
     def undo(self, num_operations: int = 1, project_path: Path = Path(".")):
         return fxt(["undo", str(num_operations), "--path", str(project_path)])
 
-    def sync(self, no_input: bool = False, project_path: Path = Path(".")):
-        return fxt(["sync", str(project_path), *_bool_flag(no_input, "no-input")])
+    def sync(
+        self,
+        prompt: bool = False,
+        quiet: bool = False,
+        project_path: Path = Path("."),
+    ):
+        return fxt(
+            [
+                "sync",
+                str(project_path),
+                *_bool_flag(prompt, "prompt"),
+                *_bool_flag(quiet, "quiet"),
+            ]
+        )
+
+    def merge_flexlate_branches(
+        self,
+        branch_name: Optional[str] = None,
+        delete: bool = True,
+        project_path: Path = Path("."),
+    ):
+        return fxt(
+            [
+                "merge",
+                *_value_if_not_none(branch_name),
+                *_bool_flag(not delete, "no-delete"),
+                "--path",
+                str(project_path),
+            ]
+        )
+
+    def push_main_flexlate_branches(
+        self,
+        remote: str = "origin",
+        project_path: Path = Path("."),
+    ):
+        return fxt(["push", "main", "--remote", remote, "--path", str(project_path)])
+
+    def push_feature_flexlate_branches(
+        self,
+        feature_branch: Optional[str] = None,
+        remote: str = "origin",
+        project_path: Path = Path("."),
+    ):
+        return fxt(
+            [
+                "push",
+                "feature",
+                *_value_if_not_none(feature_branch),
+                "--remote",
+                remote,
+                "--path",
+                str(project_path),
+            ]
+        )
 
 
 def _bool_flag(value: bool, name: str) -> List[str]:
