@@ -1,4 +1,5 @@
 # Integration tests
+import contextlib
 import os
 import shutil
 from pathlib import Path
@@ -10,8 +11,14 @@ import pytest
 from git import GitCommandError
 
 from flexlate.add_mode import AddMode
+from flexlate.branch_update import (
+    get_flexlate_branch_name,
+    get_flexlate_branch_name_for_feature_branch,
+)
 from flexlate.config import FlexlateConfig, FlexlateProjectConfig
 from flexlate.constants import DEFAULT_MERGED_BRANCH_NAME, DEFAULT_TEMPLATE_BRANCH_NAME
+from flexlate.ext_git import merge_branch_into_current
+from flexlate.main import Flexlate
 from flexlate.template.types import TemplateType
 from flexlate.template_data import TemplateData
 from tests.config import (
@@ -39,9 +46,14 @@ from tests.fixtures.template_source import (
     TemplateSourceType,
     COOKIECUTTER_REMOTE_DEFAULT_EXPECT_PATH,
 )
-from tests.gitutils import assert_main_commit_message_matches
+from tests.gitutils import (
+    assert_main_commit_message_matches,
+    checkout_new_branch,
+    checkout_existing_branch,
+)
 from tests.integration.cli_stub import CLIRunnerException
 from tests.integration.undoables import UNDOABLE_OPERATIONS
+from tests.test_pusher import add_local_remote_and_check_branches_on_exit
 
 
 def test_init_project_and_add_source_and_template(
@@ -573,15 +585,18 @@ def test_undo(
     )
     _assert_project_config_is_correct()
 
-    for branch_name in [DEFAULT_MERGED_BRANCH_NAME, DEFAULT_TEMPLATE_BRANCH_NAME]:
+    merged_branch_name = get_flexlate_branch_name(repo, DEFAULT_MERGED_BRANCH_NAME)
+    template_branch_name = get_flexlate_branch_name(repo, DEFAULT_TEMPLATE_BRANCH_NAME)
+
+    for branch_name in [merged_branch_name, template_branch_name]:
         branch = repo.branches[branch_name]  # type: ignore
         branch.checkout()
 
-    merged_branch = repo.branches[DEFAULT_MERGED_BRANCH_NAME]  # type: ignore
+    merged_branch = repo.branches[merged_branch_name]  # type: ignore
     merged_branch.checkout()
     assert_merged_commit_history_is_correct()
 
-    template_branch = repo.branches[DEFAULT_TEMPLATE_BRANCH_NAME]  # type: ignore
+    template_branch = repo.branches[template_branch_name]  # type: ignore
     template_branch.checkout()
 
     assert_main_commit_message_matches(
@@ -662,18 +677,98 @@ def test_sync_manually_remove_applied_template(
         shutil.rmtree(output_folder)
         stage_and_commit_all(repo, "Manual change to remove applied template")
 
-        fxt.sync(no_input=True)
+        fxt.sync()
+
+    merged_branch_name = get_flexlate_branch_name(repo, DEFAULT_MERGED_BRANCH_NAME)
+    template_branch_name = get_flexlate_branch_name(repo, DEFAULT_TEMPLATE_BRANCH_NAME)
 
     for branch_name in [
         "master",
-        DEFAULT_MERGED_BRANCH_NAME,
-        DEFAULT_TEMPLATE_BRANCH_NAME,
+        merged_branch_name,
+        template_branch_name,
     ]:
         branch: Head = repo.branches[branch_name]  # type: ignore
         branch.checkout()
         assert not output_folder.exists()
 
 
+def test_merge(flexlates: FlexlateFixture, repo_with_placeholder_committed: Repo):
+    fxt = flexlates.flexlate
+    repo = repo_with_placeholder_committed
+
+    with change_directory_to(GENERATED_REPO_DIR):
+        fxt.init_project()
+        # Make a dummy change
+        dummy_file = GENERATED_REPO_DIR / "something.txt"
+        dummy_file.write_text("text")
+        stage_and_commit_all(repo, "Add a dummy change to the main branch")
+        with _checkout_new_branch_that_merges_back(repo, fxt, "add-source"):
+            fxt.add_template_source(COOKIECUTTER_REMOTE_URL)
+        with _checkout_new_branch_that_merges_back(repo, fxt, "add-template"):
+            fxt.apply_template_and_add(COOKIECUTTER_REMOTE_NAME, no_input=True)
+
+    for branch_name in ["master", DEFAULT_MERGED_BRANCH_NAME]:
+        branch = repo.branches[branch_name]  # type: ignore
+        branch.checkout()
+        _assert_project_files_are_correct()
+        assert_main_commit_message_matches(
+            "Merge branch 'flexlate-templates-add-template' into flexlate-output-add-template",
+            repo.commit().message,
+        )
+
+    template_branch = repo.branches[DEFAULT_TEMPLATE_BRANCH_NAME]  # type: ignore
+    template_branch.checkout()
+    assert_main_commit_message_matches(
+        "Update flexlate templates", repo.commit().message
+    )
+
+
+def test_push(flexlates: FlexlateFixture, repo_with_placeholder_committed: Repo):
+    fxt = flexlates.flexlate
+    repo = repo_with_placeholder_committed
+
+    with change_directory_to(GENERATED_REPO_DIR):
+        fxt.init_project()
+        # Make a dummy change
+        dummy_file = GENERATED_REPO_DIR / "something.txt"
+        dummy_file.write_text("text")
+        stage_and_commit_all(repo, "Add a dummy change to the main branch")
+        with _checkout_new_branch_that_merges_back(repo, fxt, "add-source"):
+            fxt.add_template_source(COOKIECUTTER_REMOTE_URL)
+        with _checkout_new_branch_that_merges_back(
+            repo, fxt, "add-template", delete=False
+        ):
+            fxt.apply_template_and_add(COOKIECUTTER_REMOTE_NAME, no_input=True)
+
+        feature_merged_branch_name = get_flexlate_branch_name_for_feature_branch(
+            "add-template", DEFAULT_MERGED_BRANCH_NAME
+        )
+        feature_template_branch_name = get_flexlate_branch_name_for_feature_branch(
+            "add-template", DEFAULT_TEMPLATE_BRANCH_NAME
+        )
+        all_branches = [
+            DEFAULT_TEMPLATE_BRANCH_NAME,
+            DEFAULT_MERGED_BRANCH_NAME,
+            feature_merged_branch_name,
+            feature_template_branch_name,
+        ]
+
+        with add_local_remote_and_check_branches_on_exit(repo, all_branches):
+            fxt.push_feature_flexlate_branches("add-template")
+            fxt.push_main_flexlate_branches()
+
+
+@contextlib.contextmanager
+def _checkout_new_branch_that_merges_back(
+    repo: Repo, fxt: Flexlate, branch_name: str, delete: bool = True
+):
+    checkout_new_branch(repo, branch_name)
+    yield
+    fxt.merge_flexlate_branches(delete=delete)
+    checkout_existing_branch(repo, "master")
+    merge_branch_into_current(repo, branch_name)
+
+
 def _assert_project_files_are_correct(
     root: Path = GENERATED_REPO_DIR,
     expect_data: Optional[TemplateData] = None,
