@@ -8,9 +8,11 @@ import pytest
 from _pytest.monkeypatch import MonkeyPatch
 from git import Repo, Head
 
+from flexlate.branch_update import get_flexlate_branch_name_for_feature_branch
 from flexlate.config import FlexlateConfig, TemplateSource, TemplateSourceWithTemplates
 from flexlate.exc import GitRepoDirtyException
 from flexlate.finder.multi import MultiFinder
+from flexlate.pusher import Pusher
 from flexlate.render.specific import cookiecutter
 from flexlate.template.base import Template
 from flexlate.template.cookiecutter import CookiecutterTemplate
@@ -37,11 +39,14 @@ from tests.fixtures.template import *
 from tests.fixtures.templated_repo import *
 from tests.fixtures.updates import *
 from tests.fixtures.transaction import update_transaction
-from flexlate.ext_git import repo_has_merge_conflicts
-
+from flexlate.ext_git import repo_has_merge_conflicts, delete_local_branch
 
 # TODO: check that config is updated after tests
-from tests.gitutils import assert_main_commit_message_matches
+from tests.gitutils import (
+    assert_main_commit_message_matches,
+    checkout_new_branch,
+    add_local_remote,
+)
 
 
 def test_update_template_dirty_repo(
@@ -93,6 +98,74 @@ def test_update_modify_template(
     )
 
 
+def test_update_modify_template_with_feature_branches_and_main_branches_are_only_on_remote(
+    cookiecutter_one_modified_template: CookiecutterTemplate,
+    repo_with_gitignore_and_template_branch_from_cookiecutter_one: Repo,
+    update_transaction: FlexlateTransaction,
+):
+    repo = repo_with_gitignore_and_template_branch_from_cookiecutter_one
+    updater = Updater()
+    template_updates = updater.get_updates_for_templates(
+        [cookiecutter_one_modified_template], project_root=GENERATED_REPO_DIR
+    )
+
+    feature_branch = "feature"
+    checkout_new_branch(repo, feature_branch)
+    feature_merged_branch_name = get_flexlate_branch_name_for_feature_branch(
+        feature_branch, DEFAULT_MERGED_BRANCH_NAME
+    )
+    feature_template_branch_name = get_flexlate_branch_name_for_feature_branch(
+        feature_branch, DEFAULT_TEMPLATE_BRANCH_NAME
+    )
+
+    # Push to remote and delete local branches, so it will have to fetch from remote to branch off
+    add_local_remote(repo)
+    pusher = Pusher()
+    pusher.push_main_flexlate_branches(repo)
+    delete_local_branch(repo, DEFAULT_MERGED_BRANCH_NAME)
+    delete_local_branch(repo, DEFAULT_TEMPLATE_BRANCH_NAME)
+
+    updater.update(
+        repo,
+        template_updates,
+        update_transaction,
+        no_input=True,
+        merged_branch_name=feature_merged_branch_name,
+        template_branch_name=feature_template_branch_name,
+    )
+
+    # Ensure the output is correct
+    main_branch: Head = repo.branches[feature_branch]  # type: ignore
+    template_branch: Head = repo.branches[feature_template_branch_name]  # type: ignore
+    assert repo.active_branch == main_branch
+    assert (
+        repo.commit().message
+        == "Merge branch 'flexlate-templates-feature' into flexlate-output-feature\n"
+    )
+    assert_main_commit_message_matches(
+        repo.commit().parents[1].message,
+        f"Update flexlate templates\n\none: {COOKIECUTTER_ONE_MODIFIED_VERSION}",
+    )
+    assert (
+        cookiecutter_one_generated_text_content(gen_dir=GENERATED_REPO_DIR)
+        == "b and extra"
+    )
+    assert (GENERATED_REPO_DIR / "ignored" / "ignored.txt").exists()
+    assert (GENERATED_REPO_DIR / ".gitignore").exists()
+    template_branch.checkout()
+    assert repo.active_branch == template_branch
+    assert (
+        cookiecutter_one_generated_text_content(gen_dir=GENERATED_REPO_DIR)
+        == "b and extra"
+    )
+
+    # Ensure that flexlate base branches were used properly
+    base_template_branch: Head = repo.branches[DEFAULT_TEMPLATE_BRANCH_NAME]  # type: ignore
+    assert (
+        template_branch.commit.parents[0].hexsha == base_template_branch.commit.hexsha
+    )
+
+
 def test_update_modify_data(
     cookiecutter_one_template: CookiecutterTemplate,
     repo_with_gitignore_and_template_branch_from_cookiecutter_one: Repo,
@@ -123,6 +196,36 @@ def test_update_modify_data(
     assert at.data == {"a": "d", "c": "e"}
 
 
+def test_update_modify_data_for_template_in_repo(
+    cookiecutter_one_template_in_repo: CookiecutterTemplate,
+    repo_with_gitignore_and_template_branch_from_cookiecutter_one_in_repo: Repo,
+    update_transaction: FlexlateTransaction,
+):
+    repo = repo_with_gitignore_and_template_branch_from_cookiecutter_one_in_repo
+    updater = Updater()
+    template_updates = updater.get_updates_for_templates(
+        [cookiecutter_one_template_in_repo],
+        data=[{"a": "d", "c": "e"}],
+        project_root=GENERATED_REPO_DIR,
+    )
+    updater.update(repo, template_updates, update_transaction, no_input=True)
+    assert (
+        cookiecutter_one_generated_text_content(folder="d", gen_dir=GENERATED_REPO_DIR)
+        == "de"
+    )
+    assert (GENERATED_REPO_DIR / "ignored" / "ignored.txt").exists()
+    assert (GENERATED_REPO_DIR / ".gitignore").exists()
+
+    # Original b directory should no longer exist, everything moved to d
+    assert not (GENERATED_REPO_DIR / "b").exists()
+
+    config_path = GENERATED_REPO_DIR / "d" / "flexlate.json"
+    config = FlexlateConfig.load(config_path)
+    assert len(config.applied_templates) == 1
+    at = config.applied_templates[0]
+    assert at.data == {"a": "d", "c": "e"}
+
+
 def test_update_modify_template_conflict(
     cookiecutter_one_modified_template: CookiecutterTemplate,
     repo_from_cookiecutter_one_with_modifications: Repo,
@@ -133,8 +236,15 @@ def test_update_modify_template_conflict(
     template_updates = updater.get_updates_for_templates(
         [cookiecutter_one_modified_template], project_root=GENERATED_FILES_DIR
     )
-    updater.update(repo, template_updates, update_transaction, no_input=True)
-    assert repo_has_merge_conflicts(repo)
+    manual_commit_message = "Manually resolve conflicts"
+
+    def _resolve_conflicts_then_type_yes(prompt: str) -> bool:
+        assert repo_has_merge_conflicts(repo)
+        stage_and_commit_all(repo, manual_commit_message)
+        return True
+
+    with patch.object(main, "confirm_user", _resolve_conflicts_then_type_yes):
+        updater.update(repo, template_updates, update_transaction, no_input=True)
 
 
 @patch.object(
@@ -199,6 +309,58 @@ def test_update_modify_template_conflict_with_reject(
         )
 
 
+@patch.object(
+    cookiecutter, "prompt_for_config", lambda context, no_input: {"a": "b", "c": ""}
+)
+def test_update_modify_template_conflict_with_reject_on_feature_branches(
+    cookiecutter_one_modified_template: CookiecutterTemplate,
+    repo_from_cookiecutter_one_with_modifications: Repo,
+    update_transaction: FlexlateTransaction,
+):
+    repo = repo_from_cookiecutter_one_with_modifications
+
+    def _reject_update(prompt: str) -> bool:
+        return False
+
+    feature_branch = "feature"
+    checkout_new_branch(repo, feature_branch)
+
+    feature_merged_branch_name = get_flexlate_branch_name_for_feature_branch(
+        feature_branch, DEFAULT_MERGED_BRANCH_NAME
+    )
+    feature_template_branch_name = get_flexlate_branch_name_for_feature_branch(
+        feature_branch, DEFAULT_TEMPLATE_BRANCH_NAME
+    )
+
+    updater = Updater()
+    template_updates = updater.get_updates_for_templates(
+        [cookiecutter_one_modified_template], project_root=GENERATED_FILES_DIR
+    )
+    with patch.object(main, "confirm_user", _reject_update):
+        updater.update(
+            repo,
+            template_updates,
+            update_transaction,
+            merged_branch_name=feature_merged_branch_name,
+            template_branch_name=feature_template_branch_name,
+        )
+
+    assert repo.commit().message == "Prepend cookiecutter text with hello\n"
+
+    # Main branches are at original state
+    for branch_name in [DEFAULT_MERGED_BRANCH_NAME, DEFAULT_TEMPLATE_BRANCH_NAME]:
+        branch: Head = repo.branches[branch_name]  # type: ignore
+        branch.checkout()
+        assert_main_commit_message_matches(
+            repo.commit().message, "Update flexlate templates"
+        )
+
+    # Feature branches are now deleted
+    for branch_name in [feature_template_branch_name, feature_merged_branch_name]:
+        with pytest.raises(IndexError):
+            branch = repo.branches[branch_name]  # type: ignore
+
+
 @pytest.mark.parametrize(
     "remote_target_version", [None, "main", COOKIECUTTER_REMOTE_VERSION_2]
 )
